//  File SerialCamera_Mega.pde
//  Demo code for using seeeduino or Arduino Mega board to cature jpg format 
//  picturefrom seeed serial camera and save it into sd card.
//  For more details about the product please check http://www.seeedstudio.com/depot/

//  Copyright (c) 2010 seeed technology inc.
//  Author: Icing Chang
//  Version: 1.0
//  Time: Oct 27, 2010
//  Changing records:
//    
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


#include <Fat16.h>
#include <Fat16util.h> // use functions to print strings from flash memory

SdCard card;
Fat16 file;

#define CMD_LEN 11
#define PIC_BUF_LEN 64
#define SHIFT_BIT 6

#define sendCmd(cmd) {int t = 0; for(t = 0; t < CMD_LEN; t++) Serial1.print(cmd[t]);}

char cmdVGA[] = {
  0x55,0x41,0x52,0x54,0xCA,0x00,0x00,0x00,0x00,0x00,0x00};
char cmdQVGA[] = {
  0x55,0x41,0x52,0x54,0xCA,0x00,0x00,0x00,0xFF,0x00,0x00};
char cmdGetDat[] = {
  0x55,0x41,0x52,0x54,0xC7,0x00,0x00,0x00,0x00,0x00,0x00};


//byte picBuf[PIC_BUF_LEN];//buffer to temperally save pic data from camera
int picNameNum = 0;

void readCamSaveToFile(Fat16 &file, int toBeReadLen);

void setup()
{
  Serial.begin(9600);
  Serial1.begin(115200);
  // initialize the SD card
  if (!card.init()) Serial.println("card.init failed");

  // initialize a FAT16 volume
  if (!Fat16::init(&card)) Serial.println("Fat16::init failed");

}

void loop()
{
  Serial.println("\r\nplease input a character to capture a new pic!");
  while(!Serial.available());
  Serial.flush();
  Serial1.flush();
  
  //send get pic size request
  unsigned long picTotalLen = 0;
  int i = 0;
  for(i = 0; i < CMD_LEN; i++){
    Serial1.print(cmdVGA[i]);//you can change here to capture QVGA pic by using cmdQVGA[]
  }

  //get pic size response
  while(Serial1.available() < 3);
  picTotalLen |= Serial1.read()<<16;
  picTotalLen |= Serial1.read()<<8;
  picTotalLen |= Serial1.read();


  //set the name of the pic to be saved
  char picName[] = "pic00.jpg";
  picName[3] = picNameNum/10 + '0';
  picName[4] = picNameNum%10 + '0';

  //creat and open file
  // O_CREAT - create the file if it does not exist
  // O_EXCL - fail if the file exists
  // O_WRITE - open for write
  if (!file.open(picName, O_CREAT | O_WRITE)){
    Serial.println("creat file failed");
    return;
  }
  if (!file.isOpen()){
    Serial.println("file.open failed");
    return;
  }
  Serial.print(picName);
  Serial.print("    size:");
  Serial.print(picTotalLen);
  Serial.print(" bytes \r\nCapturing");
  
  //read pic data from camera and write to file in SD card
  unsigned long count = picTotalLen >> SHIFT_BIT;
  int tail = picTotalLen & (PIC_BUF_LEN - 1); 
  unsigned long addr = 0;
  cmdGetDat[5] = 0;
  cmdGetDat[6] = 0;
  cmdGetDat[7] = 0;
  cmdGetDat[10] = PIC_BUF_LEN;

  for(int i = 0; i < count; i++){//get and save count*PIC_BUF_LEN data
    sendCmd(cmdGetDat);
    readCamSaveToFile(file, PIC_BUF_LEN);
    addr += PIC_BUF_LEN;
    cmdGetDat[5] = addr >> 16;
    cmdGetDat[6] = addr >> 8;
    cmdGetDat[7] = addr;
    Serial.print('.');//you can comment it for enhancing the capture
  }

  cmdGetDat[10] = tail;//get reset of the pic data
  sendCmd(cmdGetDat);
  readCamSaveToFile(file, tail);

  // close file and force write of all data to the SD card
  if(!file.close())
  {
    Serial.print("file close error");
  }
  picNameNum++;//incease the pic number, so that the continuous pic name will changes
  Serial.println("ok");
  
}

//read the sepecific length of pic data from serial camera
//and save them to the file on the SD card
void readCamSaveToFile(Fat16 &file, int toBeReadLen)
{
  int readLen = 0;
  while(readLen < toBeReadLen)
  {
    if(Serial1.available() > 0)
    {
      byte picDat = Serial1.read();
      file.write(&picDat,1);
      //Serial.print(picDat);//for debug
      readLen++;
    }
  }
}







